#include "Character.h"
#include "Camera.h"
#include <math.h>
#include "World.h"

#define TRAVEL_SPEED (1.0f/500.0f)

Character::Character(void)
{
	Reset();
}

Character::~Character(void)
{
}


void Character::Go(int ms)
{
	Vector3d delta = Vector3d( cos(m_turnangle) * (((float)ms) * TRAVEL_SPEED),
								sin(m_turnangle) * (((float)ms) * TRAVEL_SPEED),
								0.0f);
	
	World::theWorld->ConstrainToIndoors(Position, delta);
	Position = Position + delta;

	SetLookat();
}

void Character::Strafe(int ms)
{
	Vector3d delta = Vector3d( cos(m_turnangle - HALF_PI) * (((float)ms) * TRAVEL_SPEED),
								sin(m_turnangle - HALF_PI) * (((float)ms) * TRAVEL_SPEED),
								0.0f);
	
	World::theWorld->ConstrainToIndoors(Position, delta);
	Position = Position + delta;

	SetLookat();
}

void Character::SetLookat()
{
	//find location on spherical envelope
	Lookat.x = cos(m_turnangle) * sin(m_upangle)	+ Position.x;
	Lookat.y = sin(m_turnangle) * sin(m_upangle)	+ Position.y;
	Lookat.z = cos(m_upangle)						+ Position.z;
}

void Character::ChangeLook(float dx, float dy)
{
	m_turnangle += dx; //wraps 360 degrees

	m_upangle -= dy; //artificially stop at convenient spots
	m_upangle = MAX(FOURTH_PI, m_upangle);
	m_upangle = MIN(3 * FOURTH_PI, m_upangle);

	SetLookat();
}

void Character::Reset()
{
	Camera::Reset();

	Position.x = 0.0f;
	Position.y = 0.0f;
	Position.z = 0.1f;

	m_turnangle = 0.0f;
	m_upangle = HALF_PI;

	SetLookat();
}