#include "Entity3d.h"

Entity3d::Entity3d(void)
{
	rendered_type = RENDERED_NONE;

	Connectivity.ConnectionsValue = 0;

	xpos = ypos = 0.0f;
	width = length = height = 1.0f;
}

Entity3d::~Entity3d(void)
{
}

void Entity3d::Initialize(float x, float y, float w, float l, float h)
{
	xpos = x;
	ypos = y;
	width = ABS(w);
	length = ABS(l);
	height = ABS(h);
	rendered_type = RENDERED_CUBE;
}


#define CONNECTION_WIDTH	0.3f
#define CONNECTION_LENGTH	0.5f

void Entity3d::Render(ConnectionStruct &externalConnections)
{
	//position in correct location
	glPushMatrix();

	//center
	glScalef(width, length, height);
	glTranslatef(xpos/width, ypos/length, 0);

	if(Connectivity.ConnectionsValue == 0)
	{
		glTranslatef(0,0,0.5);
		switch(rendered_type)
		{
		case RENDERED_CUBE:
			glutSolidCube(1.0);
			break;
		}
	}
	else
	{
		//central box
		glPushMatrix();
		glScalef(CONNECTION_WIDTH, CONNECTION_WIDTH, 1.0f);

		DrawTopAndBottom();

		//cap sides not in use:
		if(!Connectivity.Connections.west)
			DrawSideFace(WEST);

		if(!Connectivity.Connections.east)
			DrawSideFace(EAST);

		if(!Connectivity.Connections.north)
			DrawSideFace(NORTH);

		if(!Connectivity.Connections.south)
			DrawSideFace(SOUTH);

		glPopMatrix();

		
		if(Connectivity.Connections.east)
		{
			glPushMatrix();
			glScalef(CONNECTION_LENGTH, CONNECTION_WIDTH, 1.0f);
			glTranslatef(CONNECTION_LENGTH, 0, 0);
			DrawTopAndBottom();
			DrawSideFace(NORTH);
			DrawSideFace(SOUTH);

			if(!externalConnections.east)
				DrawSideFace(EAST);

			glPopMatrix();
		}

		if(Connectivity.Connections.west)
		{
			glPushMatrix();
			glScalef(CONNECTION_LENGTH, CONNECTION_WIDTH, 1.0f);
			glTranslatef(-CONNECTION_LENGTH, 0, 0);
			DrawTopAndBottom();
			DrawSideFace(NORTH);
			DrawSideFace(SOUTH);

			if(!externalConnections.west)
				DrawSideFace(WEST);

			glPopMatrix();
		}

		if(Connectivity.Connections.north)
		{
			glPushMatrix();
			glScalef(CONNECTION_WIDTH, CONNECTION_LENGTH, 1.0f);
			glTranslatef(0, CONNECTION_LENGTH, 0);
			DrawTopAndBottom();
			DrawSideFace(EAST);
			DrawSideFace(WEST);

			if(!externalConnections.north)
				DrawSideFace(NORTH);

			glPopMatrix();
		}

		if(Connectivity.Connections.south)
		{
			glPushMatrix();
			glScalef(CONNECTION_WIDTH, CONNECTION_LENGTH, 1.0f);
			glTranslatef(0, -CONNECTION_LENGTH, 0);
			DrawTopAndBottom();
			DrawSideFace(EAST);
			DrawSideFace(WEST);

			if(!externalConnections.south)
				DrawSideFace(SOUTH);

			glPopMatrix();
		}
	}

	glPopMatrix();
}

void Entity3d::DrawFace(float yangle, float zangle)
{
	glPushMatrix();
	
	glRotatef(yangle, 0, 1.0f, 0); 
	glRotatef(zangle, 1.0f, 0, 0);

	glTranslatef(0,0,+0.5);

	glBegin(GL_QUADS);
	glTexCoord2f(0.0f, 0.0f); glVertex3f(-0.5f,	-0.5f, 0.0f);
	glTexCoord2f(1.0f, 0.0f); glVertex3f(0.5f, -0.5f, 0.0f);
	glTexCoord2f(1.0f, 1.0f); glVertex3f(0.5f, 0.5f, 0.0f);
	glTexCoord2f(0.0f, 1.0f); glVertex3f(-0.5f, 0.5f, 0.0f);
	glEnd();

	glPopMatrix();
}

void Entity3d::DrawTopAndBottom()
{
	DrawFace(0);
	DrawFace(180);
}

void Entity3d::DrawSideFace(Direction dir)
{
	switch(dir)
	{
	case WEST:
		DrawFace(90, 180);
		break;
	case EAST:
		DrawFace(-90, 180);
		break;
	case NORTH:
		DrawFace(-90, -90);
		break;
	case SOUTH:
		DrawFace(90, 90);
		break;
	}
}