#pragma once

#define MAX_RENDERABLE 20

#include <utils.h>
#include <string>
#include "WindowMapping.h"

#include "Vector3d.h"




/*
 * Manages all layers as renderables, and as providers.
*/

class LayerManager
{
public:
	//please call this first.
	void Initialize(GLUI* parentGlui);
	void SetViewportSize(int w, int h);

	DisplayMode m_renderMode;

	bool ShouldUpdateLayers();

	void Translate(float delta_x, float delta_y);
	void Scale(float scale_x, float scale_y);

	void GetWorldCoordinates(float &x, float &y);

	void ChangeParameter(float newParameter);
	void IncrementParameter(float delta);

	void RemoveAllLayers();

	void UpdateTextures(bool force = false);
	void Render();

	LayerManager(void);
	~LayerManager(void);

	//Layer Management:
	void SetDefaults();
	void AddLayer(BaseRenderable* newLayer);
	void RemoveLayer(int id = -1);

	BaseRenderable* m_renderableList[MAX_RENDERABLE];
	int m_renderableCount;

	//only one - singleton-like
	static LayerManager *m_Manager;
	int m_width, m_height;

private:

	WindowMapping m_mapping2d;

	void UpdateWindowMapper();
	void SetSelectedLayer();
	
	//signals the main loop to redraw the viewport.
	bool m_shouldUpdateLayers;

	void SetVariableOverrideValue();

	BaseRenderable *m_currentLayer;
	//int m_selectedLayer;

	int GetIdIndex(int id);

	void SetGuiInformation();
	void SetSelectedVariable();
	void SetInputLayer();
	void RemoveInputLayer();

	void InverseChecked();
	void SetWindowSize();

	bool IsCurrentLayer3d();

	void NavigateInto();
	void NavigateUp();
	void SelectVisualMode();
	void SelectRenderMode();

	static void event_InverseCheck(int value);
	static void event_WindowsizeSpinner(int value);

	static void event_RemoveLayer(int value);
	static void event_AddLayer(int value);
	static void event_ChangeSelection(int value);
	static void event_SetInputLayer(int value);
	static void event_loadXML(int value);
	static void event_saveXML(int value);
	static void event_defaults(int value);
	static void event_visualmode(int value);
	static void event_rendermode(int value);
	

	static void event_VariableChangeSelection(int value);
	static void event_RemoveInputLayer(int value);
	static void event_NavigateInto(int value);
	static void event_NavigateUp(int value);

	void LoadFromXml();
	XmlError LoadFromXmlWithErrors(string path = "");
	void SaveToXml();

	GLUI *m_parentGlui;
	
	GLUI_Panel *m_panelLayers, *m_layerVariables, *m_panelRendermode;
	
	GLUI_Listbox 
		*m_lbLayers,
		*m_lbVariables, *m_lbLayerTypes, *m_lbEntityTypes;

	GLUI_EditText *m_txtVariableValue;                                                           

	GLUI_Checkbox *m_cbInverse;

	GLUI_Button
		*m_btnRemoveLayer, 
		*m_btnAddLayer,
		*m_btnSetInputLayer, *m_btnRemoveInputLayer,
		*m_btnNavigateInto, *m_btnNavigateUp,
		*m_btnLoadXML, *m_btnSaveXML,
		*m_btnLoadDefaults;

	GLUI_Spinner *m_3dWindow;

	GLUI_RadioGroup *m_grpVisualMode;
	GLUI_RadioButton *m_radTopDown, *m_radCharacter;

	GLUI_RadioGroup *m_grpRenderMode;
	GLUI_RadioButton *m_radSolid, *m_radWireFrame;
};