#include "Renderable2d.h"
#include "LayerManager.h"
#include "BaseRenderable.h"

//#include "../../gauss/gaussianblur.h"

Renderable2d::Renderable2d(int TypeID) : BaseRenderable(TypeID)
{
	m_inverse = false;
}

Renderable2d::~Renderable2d(void)
{
}

void Renderable2d::Initialize()
{

}

//#define FIND_MINMAX

void Renderable2d::WriteBitmap()
{
	//creating 2d bitmap texture.
	GLubyte gray;

#ifdef FIND_MINMAX
	float min = 0.5f, max = 0.0f;
#endif

	for(int i = 0; i < m_layerManager->m_width; i++)
	{
		for(int j = 0; j < m_layerManager->m_height; j++)
		{
			float x, y; x = i; y = j;
			m_layerManager->GetWorldCoordinates(x, y);
			float value = GetValueAt(x, y);
			gray = (GLubyte) 0xFF * value;

#ifdef FIND_MINMAX
			if(value < min) min = value;
			if(value > max) max = value;
#endif

			m_bitmapBits[i + m_layerManager->m_width*j] = gray;

		}
	}

#ifdef FIND_MINMAX
	LOG(L"Min: %g, max: %g", min, max);
#endif


//	TGaussianBlur<GLubyte> BlurFilter;
//	BlurFilter.Filter(m_bitmapBits, NULL, m_layerManager->m_width, m_layerManager->m_height, 51);
}


void Renderable2d::Update(bool force)
{
	// write to the texture
	WriteBitmap();

	glBindTexture(GL_TEXTURE_2D, m_Texture);
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);	// Linear Min Filter
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);	// Linear Mag Filter

	if(!m_bitmapBits)
	{
		//I don't think this part gets called - because we never recreate the bitmap.
		//resizing the window SHOULD do that, eventually.
		GLubyte gray = 0x33;
		glTexImage2D(GL_TEXTURE_2D, 0, 1, 1, 1, 0, GL_LUMINANCE, GL_UNSIGNED_BYTE, &gray);
	}
	else
		glTexImage2D(GL_TEXTURE_2D, 0, 1, m_layerManager->m_width, m_layerManager->m_height, 0, GL_LUMINANCE, GL_UNSIGNED_BYTE, m_bitmapBits);

}

void Renderable2d::Render()
{
	glColor4f(1.0f,1.0f,1.0f, 1.0f);
	glBindTexture(GL_TEXTURE_2D, m_Texture);

	glBegin(GL_QUADS);
		// Front Face
		glTexCoord2f(0.0f, 0.0f); glVertex3f(0,		0,    0.0f);
		glTexCoord2f(1.0f, 0.0f); glVertex3f(1.0f,	0.0f, 0.0f);
		glTexCoord2f(1.0f, 1.0f); glVertex3f(1.0f,  1.0f,  0.0f);
		glTexCoord2f(0.0f, 1.0f); glVertex3f(0.0f,  1.0f,  0.0f);
	glEnd();
}

float Renderable2d::GetValueAt(float x, float y)
{
	//transform to this sub-layer
	m_transform.Apply(x, y);

	//perform further standard checks
	float value = GetRawValueAt(x, y);
	if(m_inverse)
		return (1.0 - value);

	return value;
}