#include "Renderable3d.h"

GLubyte SomeColors[5][3] = {
	{0x88, 0, 0},
	{0, 0x88, 0},
	{0, 0, 0x88},
	{0x44, 0x44, 0},
	{0x44, 0, 0x44}
};

Renderable3d::Renderable3d(int typeID) : BaseRenderable(typeID)
{
	m_WindowSizeX = m_WindowSizeY = 3;
	world = World::theWorld;
	m_entityCollection.Clear();
	for(int i = 0; i < TEXTURE_COUNT; i++)
	{
		//generate textures
		glGenTextures(TEXTURE_COUNT, m_entityTextures);	
	}

	m_firstEntityList = glGenLists(TEXTURE_COUNT);

	SetSpaceWindow();
}

void Renderable3d::Render()
{
	//ONLY draw, ffs.
	m_entityCollection.Render(m_entityTextures, m_firstEntityList);
}

void Renderable3d::Update(bool force)
{
	bool changedWindow = SetSpaceWindow();
	if(!changedWindow && !force) return;

	//set texture
	for(int i = 0; i < TEXTURE_COUNT; i++)
	{
		glBindTexture(GL_TEXTURE_2D, m_entityTextures[i]);
		glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);	// Linear Min Filter
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);	// Linear Mag Filter

		glTexImage2D(GL_TEXTURE_2D, 0, 3, 1, 1, 0, GL_RGB, GL_UNSIGNED_BYTE, SomeColors[i]);

	}

	//building 3d entity collection
	BuildCollection();

	//Updating render lists
	m_entityCollection.CompileDisplayLists(m_firstEntityList);

}

bool Renderable3d::SetSpaceWindow()
{
	//keep divisible by 2
	if(m_WindowSizeX % 2 != 0)
		m_WindowSizeX++;

	if(m_WindowSizeY % 2 != 0)
		m_WindowSizeY++;

	//find size of space window
	float x_width = m_WindowSizeX * GRID_SIZE;
	float y_width = m_WindowSizeY * GRID_SIZE;

	int x, y;
	world->GetWindowCenter(x, y);

	WindowMapping tempWindow;
	tempWindow.SetPhysicalWindow(m_WindowSizeX, m_WindowSizeY);
	tempWindow.SetWorldWindow(-y_width/2 + y*GRID_SIZE, x_width/2 + x*GRID_SIZE, -x_width/2 + x*GRID_SIZE, y_width/2 + y*GRID_SIZE);

	//no changes - nothing should happen. Window is snapped to grid points.
	if(tempWindow == m_spaceWindow)
		return false;

	//copy constructor
	m_spaceWindow = tempWindow;
	return true;
}

bool Renderable3d::SetWindowSize(int newWindowSize)
{
	if(newWindowSize >=2 && m_WindowSizeX != newWindowSize)
	{
		m_WindowSizeX = m_WindowSizeY = newWindowSize;
		return true;
	}

	return false;
}