#include "World.h"

World* World::theWorld = 0;

#define SPACE_WINDOW_MARGIN ((float)1/8)

World::World(void)
{
	m_visualmode = VISUAL_TOPDOWN;
	m_rendermode = RENDER_SOLID;
	theWorld = this;

	memset(KeyDown, 0, sizeof(KeyDown));
}

World::~World(void)
{
}

void World::ResetCameras()
{
	camera.Reset();
	character.Reset();
}

VisualMode World::GetVisualMode()
{
	return m_visualmode;
}

void World::SetVisualMode(VisualMode selected)
{
	m_visualmode = selected;
}

RenderMode World::GetRenderMode()
{
	return m_rendermode;
}

void World::SetRenderMode(RenderMode selected)
{
	m_rendermode = selected;
}

void World::CreateView()
{
	switch(m_visualmode)
	{
	case VISUAL_TOPDOWN:
		camera.CreateCamera();
		break;
	case VISUAL_CHARACTER:
		character.CreateCamera();
		break;
	}

	switch(m_rendermode)
	{
	case RENDER_SOLID:
		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
		break;
	case RENDER_WIREFRAME:
		glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
		break;
	}
}

void World::GetGridContainingPoint(const Vector3d &point, int &x, int &y)
{
	GetGridContainingPoint(point.x, point.y, x, y);
}

void World::GetGridContainingPoint(float xpos, float ypos, int &x, int &y)
{
	x = DivFloat(xpos, GRID_SIZE);
	y = DivFloat(ypos, GRID_SIZE);
}

void World::GetWindowCenter(int &x, int &y)
{
	switch(m_visualmode)
	{
	case VISUAL_TOPDOWN:
		{
			GetGridContainingPoint(camera.Lookat.x, camera.Lookat.y, x, y);
			return;
		}
		break;
	case VISUAL_CHARACTER:
		{
			GetGridContainingPoint(character.Position.x, character.Position.y, x, y);
			return;
		}
		break;
	}
}

void World::PanMouse(float dx, float dy)
{
	switch(m_visualmode)
	{
	case VISUAL_TOPDOWN:
		{
			dx *= 10;
			dy *= 10;
			Vector3d thevector = ( camera.Lookat - camera.Position).Normalize();
			if(dx != 0)
			{
				camera.Lookat.x += thevector.y * dx;
				camera.Lookat.y -= thevector.x * dx;

				camera.Position.x += thevector.y * dx;
				camera.Position.y -= thevector.x * dx;
			}
			if(dy != 0)
			{
				camera.Lookat.x -= thevector.x * dy;
				camera.Lookat.y -= thevector.y * dy;

				camera.Position.x -= thevector.x * dy;
				camera.Position.y -= thevector.y * dy;
			}
		}
		break;
	case VISUAL_CHARACTER:
		{
			character.ChangeLook(dx, dy);
		}
		break;
	}

}

void World::ZoomMouse(float dy)
{
	switch(m_visualmode)
	{
	case VISUAL_TOPDOWN:
		{
			Vector3d outdirection = (camera.Position - camera.Lookat);
			float currentmag = outdirection.Magnitude();
			float newmag = currentmag + dy*10;
			newmag = MAX(newmag, MINIMUM_MAGNITUDE);

			camera.Position = (outdirection.Normalize() * newmag) + camera.Lookat;
		}
		break;
	case VISUAL_CHARACTER:
		{
			//nothing for now, maybe
			break;
		}
	}
}

void World::BuildLighting()
{
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glEnable(GL_LIGHT1);

	//setting intensity
	float topintensity = 0.5f;
	GLfloat paramsIntensity[] = {topintensity, topintensity, topintensity};
	glLightfv(GL_LIGHT1, GL_DIFFUSE, paramsIntensity);

	//setting position
	GLfloat paramsPosition[] = {camera.Position.x, camera.Position.y, camera.Position.z, 1};
	glLightfv(GL_LIGHT0, GL_POSITION, paramsPosition);

	glEnable(GL_COLOR_MATERIAL);
	glColorMaterial ( GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE ) ;

	if(m_visualmode == VISUAL_CHARACTER)
	{
		//add another light
		glEnable(GL_LIGHT2);

		//setting intensity
		float charintensity = 0.3f;
		GLfloat paramsIntensity2[] = {charintensity, charintensity, charintensity};
		glLightfv(GL_LIGHT2, GL_DIFFUSE, paramsIntensity2);

		GLfloat paramsPosition2[] = {character.Position.x, character.Position.y, character.Position.z * 3, 1.0f};
		glLightfv(GL_LIGHT2, GL_POSITION, paramsPosition2);
	}
}

bool World::DoFrame(int timeElapsed)
{
	bool didChange = false;

	if(m_visualmode == VISUAL_CHARACTER)
	{
		//check keyboard input
		if(KeyDown['w'])
		{
			//forward
			character.Go(timeElapsed);
			didChange = true;
		}
		else if(KeyDown['s'])
		{
			//backward
			character.Go(-timeElapsed);
			didChange = true;
		}

		//separately
		if(KeyDown['a'])
		{
			//left
			character.Strafe(-timeElapsed);
			didChange = true;
		}
		else if(KeyDown['d'])
		{
			//right
			character.Strafe(timeElapsed);
			didChange = true;
		}

		if(didChange)
		{
			//make topdown camera follow the character.
			Vector3d diff = character.Position - camera.Lookat;
			camera.Lookat = camera.Lookat + diff;
			camera.Position = camera.Position + diff;
		}
	}

	return didChange;
}

bool World::ConstrainToIndoors(Vector3d &originalPoint, const Vector3d &delta)
{
	int gridx, gridy;
	GetGridContainingPoint(originalPoint, gridx, gridy);

	return true;

}