#pragma once

#include <utils.h>
#include "Camera.h"
#include "Serializable.h"
#include "Character.h"

#define GRID_SIZE 1.0f
#define MINIMUM_MAGNITUDE 1.0f

enum RenderMode
{
	RENDER_SOLID,
	RENDER_WIREFRAME
};

enum VisualMode
{
	VISUAL_TOPDOWN,
	VISUAL_CHARACTER
};

class World: public Serializable
{
public:
	//singleton object
	static World* theWorld;

	VisualMode GetVisualMode();
	void SetVisualMode(VisualMode selected);

	RenderMode GetRenderMode();
	void SetRenderMode(RenderMode selected);

	void ResetCameras();

	void PanMouse(float dx, float dy);
	void ZoomMouse(float dy);
	bool DoFrame(int timeElapsed);

	void GetWindowCenter(int &x, int &y);

	void GetGridContainingPoint(float xpos, float ypos, int &x, int &y);
	void GetGridContainingPoint(const Vector3d &point, int &x, int &y);

	void BuildLighting();
	void CreateView();

	void SaveToXml(TiXmlElement *currentElement);
	void LoadFromXml(TiXmlElement *currentElement);

	bool ConstrainToIndoors(Vector3d &originalPoint, const Vector3d &delta);

	World(void);
	~World(void);

	//keys that are down.
	//methodology borrowed from http://www.nullterminator.net/glut.html
	int KeyDown[256];

private:
	VisualMode m_visualmode;
	RenderMode m_rendermode;

	Camera camera;
	Character character;
};